@@ -3,17 +3,20 @@ package XML::PP;
 use strict;
 use warnings;
 
+use Log::Abstraction;
+use Scalar::Util;
+
 =head1 NAME
 
 XML::PP - A simple XML parser
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -47,6 +50,7 @@ This module supports basic XML document parsing, including namespace handling, a
   my $parser = XML::PP->new(warn_on_error => 1);
 
 Creates a new C<XML::PP> object.
+It can take several optional arguments:
 
 =over 4
 
@@ -54,6 +58,15 @@ Creates a new C<XML::PP> object.
 
 =item * C<warn_on_error> - If true, the parser emits warnings for unknown or malformed XML entities. This is enabled automatically if C<strict> is enabled.
 
+=item * C<logger>
+
+Used for warnings and traces.
+It can be an object that understands warn() and trace() messages,
+such as a L<Log::Log4perl> or L<Log::Any> object,
+a reference to code,
+a reference to an array,
+or a filename.
+
 =back
 
 =cut
@@ -67,10 +80,19 @@ sub new {
 		$opts{warn_on_error} = 1;
 	}
 
-	return bless {
+	my $self = bless {
 		strict => $opts{strict} // 0,
 		warn_on_error => $opts{warn_on_error} // 0,
+		%opts,
 	}, $class;
+
+	if(my $logger = $self->{'logger'}) {
+		if(!Scalar::Util::blessed($logger)) {
+			$self->{'logger'} = Log::Abstraction->new($logger);
+		}
+	}
+
+	return $self;
 }
 
 =head2 parse
@@ -105,9 +127,11 @@ sub parse {
 		# $self->_handle_error("Invalid or empty XML document provided");
 	if (!$xml_string) {
 		# $self->_handle_error("Empty XML document provided");
-		return;
+		return {};
 	}
 
+	$xml_string =~ s/<\?xml.+?>//;	# Ignore the header
+
 	$xml_string =~ s/^\s+|\s+$//g;	# Trim whitespace
 	return $self->_parse_node(\$xml_string, {});
 }
@@ -368,14 +392,28 @@ sub _decode_entities {
 
 sub _handle_error {
 	my ($self, $message) = @_;
+
 	my $error_message = __PACKAGE__ . ": XML Parsing Error: $message";
 
-	if ($self->{strict}) {
-		die $error_message;	# Throws an error if strict mode is enabled
+	if($self->{strict}) {
+		# Throws an error if strict mode is enabled
+		if($self->{'logger'}) {
+			$self->fatal($error_message);
+		}
+		die $error_message;
 	} elsif ($self->{warn_on_error}) {
-		warn $error_message;	# Otherwise, just warn
+		# Otherwise, just warn
+		if($self->{'logger'}) {
+			$self->warn($error_message);
+		} else {
+			warn $error_message;
+		}
 	} else {
-		print STDERR "Warning: $error_message\n";
+		if($self->{'logger'}) {
+			$self->notice($error_message);
+		} else {
+			print STDERR "Warning: $error_message\n";
+		}
 	}
 }
 
